#' Convert GFF to a matrix
#'
#' This function takes a GFF data frame and converts it into a matrix
#'
#' @param input A data frame containing GFF data.
#'              Typically, this data frame is generated by importing a GFF file using `rtracklayer::import.gff`
#'              and converting it to a data frame
#' @return A matrix, where values are binary: `1` indicates the presence of a feature in a sequence,
#'          and `0` indicates its absence
#' @examples
#' gff_file <- system.file("extdata/out_Hb_gff.txt", package = "PMScanR")
#' if (gff_file != "") {
#'   gff_data <- rtracklayer::import.gff(gff_file)
#'   gff_data <- as.data.frame(gff_data)
#'   matrix <- gff2matrix(gff_data)
#'   print(matrix)
#' }
#' @importFrom reshape2 dcast
#' @export
gff2matrix <- function(input) {
  input$matchID <- paste0(input$type, ":", input$start, "-", input$end)
  input$matchName <- if ("Name" %in% colnames(input)) {
    paste0(input$Name, ":", input$start, "-", input$end)
  } else {
    input$matchID
  }
  at <- input[c("seqnames", "matchID")]
  at$exist <- 1
  df_at <- reshape2::dcast(at, matchID ~ seqnames, value.var = "exist", fill = 0)
  rownames(df_at) <- df_at$matchID
  df_at <- df_at[, -1, drop = FALSE]  # Remove matchID column, keep as matrix
  return(as.matrix(df_at))
}
