## ----setup, include=FALSE-----------------------------------------------------
# Standard setup chunk
knitr::opts_chunk$set(echo = TRUE, collapse = TRUE)
# Ensure necessary libraries are loaded for the vignette build process itself
# (Users need to install them separately to run the code interactively)
library(PMScanR)
library(ggseqlogo)
library(seqinr) # Seems to be used in extract_segments example

## ----installation-loading, eval=FALSE-----------------------------------------
# # Installation command - typically not evaluated in a vignette
# # User should install before using the vignette
# # devtools::install_github("prodakt/PMScanR", force = TRUE)
# 
# # Load libraries
# library(PMScanR)
# library(ggseqlogo)

## ----run-shiny-app, eval=FALSE------------------------------------------------
# # Running Shiny apps is interactive, usually set eval=FALSE in vignettes
# # runPMScanRShiny()

## ----set-working-directory, eval=FALSE----------------------------------------
# # Setting working directory is user-specific, usually eval=FALSE in vignettes
# # setwd("disc:/your/path/to/working/directory")

## ----load-example-fasta-------------------------------------------------------
# Load example FASTA file included with the package
fasta_file <- system.file("extdata", "hemoglobins.fasta", package = "PMScanR")
# Check if file exists (optional, good practice)
if (!file.exists(fasta_file)) {
  warning("Example FASTA file not found. Vignette examples may fail.")
}

## ----run-ps-scan-auto, eval=FALSE---------------------------------------------
# # This requires ps_scan to be installed and configured,
# # and might download data, so often set eval=FALSE in automated checks/vignettes.
# # Set eval=TRUE if you have a small example and ps_scan configured in the build environment.
# # runPsScan(in_file = fasta_file, out_format = 'gff', out_file = "results_pfscan.gff")

## ----define-prosite-output-path, eval=FALSE-----------------------------------
# ## Path to an actual output file (user specific)
# # motifs_prosite <- "out_Hb_psa.txt"

## ----define-psa-output-path, eval=FALSE---------------------------------------
# # Path to an actual output file (user specific)
# # motifs_psa <- "out_Hb_psa.txt"

## ----read-example-psa---------------------------------------------------------
# Load example PSA output file included with the package
motifs_psa_file <- system.file("extdata", "out_Hb_psa.txt", package = "PMScanR")

# Check if file exists
if (file.exists(motifs_psa_file)) {
  psaGFF <- read.psa(motifs_psa_file)
  # Optional: display structure or head if needed
  # str(psaGFF)
  # head(psaGFF)
} else {
  warning("Example PSA file not found. Downstream examples may fail.")
  # Create a placeholder empty data frame if needed for vignette to build
  psaGFF <- data.frame(seqid=character(), source=character(), type=character(),
                       start=integer(), end=integer(), score=numeric(),
                       strand=character(), phase=character(), attributes=character(),
                       stringsAsFactors=FALSE)
}


## ----convert-gff-to-matrix----------------------------------------------------
# 'psaGFF' is the GFF-like data frame from PSA conversion
# Check if psaGFF has data before proceeding
if (nrow(psaGFF) > 0) {
 mom <- gff2matrix(psaGFF)
 # Display the first few rows of the Motif Occurrence Matrix (MOM)
 head(mom)
} else {
 warning("Input GFF data is empty. Cannot create matrix.")
 mom <- matrix(nrow=0, ncol=0) # Placeholder
}

## ----generate-heatmap-1, fig.show='hold'--------------------------------------
# This example creates a heatmap from the MOM derived from PSA-converted GFF data ('mom').
# Check if mom has data
if (nrow(mom) > 0 && ncol(mom) > 0) {
  hm1 <- matrix2hm(x = colnames(mom), # 'x' argument specifies columns (motifs)
                   y = row.names(mom), # 'y' argument specifies rows (sequences)
                   input = mom)        # 'input' argument takes the MOM
  # Display the heatmap 'hm1' (might require explicit print for plotly in vignettes)
  print(hm1)
} else {
  warning("Motif matrix is empty. Cannot generate heatmap.")
}

## ----generate-heatmap-2, fig.show='hold'--------------------------------------
# Example of usage for creating a heatmap using matrix2hm_2() - from PSA-converted Matrix:
# This example uses 'matrix2hm_2()' to visualize the MOM from PSA-converted data ('mom').
# Check if mom has data
if (nrow(mom) > 0 && ncol(mom) > 0) {
  hm2 <- matrix2hm_2(x = colnames(mom),
                     y = row.names(mom),
                     input = mom)
  # Display heatmap 'hm2'
  print(hm2)
} else {
  warning("Motif matrix is empty. Cannot generate heatmap 2.")
}

## ----extract-segments-from-fasta, fig.show='hold'-----------------------------
# Starting position of the region
from_pos <- 10
# Ending position of the region
to_pos <- 20

# Read the FASTA file containing protein sequences (loaded earlier)
# Check if fasta_file path is valid and file exists
if (exists("fasta_file") && file.exists(fasta_file)) {
  seq <- seqinr::read.fasta(file = fasta_file, seqtype = "AA")
  # Extract segments from position 'from_pos' to 'to_pos' for all sequences
  # Ensure the function exists and seq has content
  if (length(seq) > 0) {
     seqShort <- extract_segments(seq = seq, from = from_pos, to = to_pos)
     # Check if seqShort has content before plotting
     if (length(unlist(seqShort)) > 0) {
        ggseqlogo::ggseqlogo(unlist(seqShort), seq_type = "aa")
     } else {
        warning("No segments extracted, cannot generate seqlogo.")
     }
  } else {
     warning("FASTA sequence data is empty.")
  }
} else {
  warning("Example FASTA file path not found or invalid.")
}


## ----extract-motifs-from-psa, fig.show='hold'---------------------------------
# Check if motifs_psa_file exists
if (exists("motifs_psa_file") && file.exists(motifs_psa_file)) {
  protein_motifs_psa <- extract_protein_motifs(motifs_psa_file)
  # Check results before plotting
  if (length(protein_motifs_psa) >= 1) {
     ggseqlogo::ggseqlogo(protein_motifs_psa[[1]], seq_type = 'aa')
  } else {
     warning("No motifs extracted or result format unexpected (expected list).")
  }
  if (length(protein_motifs_psa) >= 5) {
     ggseqlogo::ggseqlogo(protein_motifs_psa[[5]], seq_type = 'aa')
  } else {
     warning("Fewer than 5 motifs extracted.")
  }
} else {
  warning("Example PSA file path not found or invalid.")
}


## ----generate-pie-chart, eval=FALSE, fig.show='hold'--------------------------
# # Example usage would require the psaGFF object from earlier
# # Check if psaGFF has data
# # if (exists("psaGFF") && nrow(psaGFF) > 0) {
# #   # Assuming freqPie takes the GFF data frame as input
# #   pie_chart <- freqPie(psaGFF)
# #   print(pie_chart)
# # } else {
# #   warning("GFF data is empty. Cannot generate pie chart.")
# # }
# # Setting eval=FALSE as freqPie function definition/usage isn't fully clear from text

## ----session-info-------------------------------------------------------------
sessionInfo()

